"use strict";
exports.__esModule = true;
function memoize(name) {
    var selectorsAndResolver = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        selectorsAndResolver[_i - 1] = arguments[_i];
    }
    if (selectorsAndResolver.length < 2) {
        throw new Error('At least a selector and a resolver must be provided.');
    }
    selectorsAndResolver.forEach(function (arg) {
        if (typeof arg !== 'function') {
            throw new Error('Selectors and resolvers must be functions');
        }
    });
    if (!name || typeof name !== 'string') {
        throw new Error(name + " is not a valid prop name");
    }
    var resolver = selectorsAndResolver[selectorsAndResolver.length - 1];
    var selectors = selectorsAndResolver.slice(0, -1);
    var injector = {};
    injector.initialStateInjector = function (props, context, state) {
        var args = selectors.map(function (s) { return s(props, context); });
        state.value = resolver.apply(void 0, args);
    };
    injector.componentWillReceivePropsHook = function (nextProps, nextContext, props, context, state) {
        var modified = false;
        var args = selectors.map(function (s) {
            var value = s(nextProps, nextContext);
            if (!modified && s(props, context) !== value) {
                modified = true;
            }
            return value;
        });
        if (modified) {
            state.value = resolver.apply(void 0, args);
        }
    };
    injector.propInjector = function (setProp, _p, _c, state) {
        setProp(name, state.value);
    };
    return injector;
}
exports["default"] = memoize;
function context(name) {
    return {
        contextTypeInjector: function (setContextType) { return setContextType(name, function () { return null; }); }
    };
}
exports.context = context;
